@@ -57,12 +57,6 @@ extern Gatefunc sci_gf_scilab;
 extern Gatefunc sci_gf_scilab;
 extern Gatefunc sci_gf_scilab;
 extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
-extern Gatefunc sci_gf_scilab;
 extern Gatefunc sci_spluinc;
 extern Gatefunc sci_splu;
 extern Gatefunc sci_splusolve;
@@ -98,12 +92,9 @@ static GenericTable Tab[]={
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im_get"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im_set"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdbrick"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdbrick_get"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdbrick_set"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdstate"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdstate_get"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mdstate_set"},
+  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im_data"},
+  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im_data_get"},
+  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_mesh_im_data_set"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_model"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_model_get"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_model_set"},
@@ -128,9 +119,6 @@ static GenericTable Tab[]={
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_exit"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_cont_struct_get"},
   {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_cont_struct"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_multi_contact_frame_get"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_multi_contact_frame_set"},
-  {(Myinterfun)sci_gateway,sci_gf_scilab,"gf_multi_contact_frame"},
   {(Myinterfun)sci_gateway,sci_spluinc,"sp_luinc"},
   {(Myinterfun)sci_gateway,sci_splu,"sp_lu"},
   {(Myinterfun)sci_gateway,sci_splusolve,"sp_lusolve"},
