@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 1999-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 1999-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file bgeot_config.h
@@ -51,6 +51,7 @@
 #include "gmm/gmm_dense_lu.h"
 
 #ifdef GETFEM_HAVE_BOOST
+# include <boost/version.hpp>
 # include <boost/noncopyable.hpp>
 #else
 # include <getfem_boost/noncopyable.hpp> 
@@ -59,9 +60,9 @@
 #ifdef GETFEM_HAVE_QDLIB
 // #  define NO_INLINE
 #  ifdef GETFEM_QDLIB_USE_QUAD
-#    include <qd/qd.h>
+#    include <qd/qd_real.h>
 #  else
-#    include <qd/dd.h>
+#    include <qd/dd_real.h>
 #  endif
 #  include <qd/fpu.h>
 #endif
@@ -74,12 +75,14 @@ namespace bgeot {
   
 
   static const size_t ST_NIL = size_t(-1);
-  /// Dimension type (<255)
-  typedef gmm::uint8_type dim_type;
+  typedef gmm::uint16_type dim_type;
   typedef gmm::uint16_type short_type;
   typedef size_t size_type;
   typedef double scalar_type;
   typedef std::complex<double> complex_type;
+  inline double to_double(double &a) { return a; }
+  inline scalar_type to_scalar(const scalar_type &a) { return a; }
+
 #ifndef GETFEM_HAVE_QDLIB
   typedef double long_scalar_type;
   typedef double opt_long_scalar_type;
@@ -90,17 +93,25 @@ namespace bgeot {
 #  ifdef GETFEM_QDLIB_USE_QUAD
   typedef qd_real long_scalar_type;
   typedef qd_real opt_long_scalar_type;
+  inline scalar_type to_scalar(const qd_real &a) { return to_double(a); }
 # define LONG_SCALAR_ATOF(st) (long_scalar_type(st))
 # define LONG_SCALAR_EPS 1E-64
 #  else
   typedef dd_real long_scalar_type;
   typedef dd_real opt_long_scalar_type;
+  inline scalar_type to_scalar(const dd_real &a) { return to_double(a); }
 # define LONG_SCALAR_ATOF(st) (long_scalar_type(st))
 # define LONG_SCALAR_EPS 1E-32
 #  endif
 #  define LONG_SCAL(xx) long_scalar_type(#xx) /* string assignment to preserve the precision */
 #endif
 
+  typedef std::vector<scalar_type> base_vector;
+  typedef std::vector<complex_type> base_complex_vector;
+  typedef gmm::dense_matrix<scalar_type> base_matrix;
+  typedef gmm::dense_matrix<complex_type> base_complex_matrix;
+
+
   // For compatibility with Getfem 2.0
 
   using gmm::dimension_error;
