@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 1995-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 1995-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 
@@ -51,6 +51,8 @@
 #include "dal_basic.h"
 #include <limits.h>
 #include <bitset>
+#include <iterator>
+#include <algorithm>
 
 namespace dal {
 
@@ -214,6 +216,7 @@ namespace dal {
     const_iterator begin(void) const { return const_iterator(*this, 0); }
     iterator end(void) { return iterator(*this, size()); }
     const_iterator end(void) const { return const_iterator(*this, size()); }
+
     reverse_iterator rbegin(void) { return reverse_iterator(end()); }
     const_reverse_iterator rbegin(void) const
     { return const_reverse_iterator(end()); }
@@ -318,7 +321,7 @@ namespace dal {
     inline int take_last(void)
     { int res = last(); if (res >= 0) sup(res); return res; }
   };
-  
+
   /**
      if you are only interested in indexes of true values of a bit_vector
      (i.e. if you use it as an int set), use bv_visitor instead of
@@ -376,6 +379,102 @@ namespace dal {
   { s.add(i); return i; }
 
   std::ostream APIDECL &operator <<(std::ostream &o, const bit_vector &s);
+  
+  /**Iterator class for bv_iterable and bv_iterable_c.*/
+  template<typename ITERABLE_BV>
+  class const_bv_iterator
+  {
+    typedef dal::bit_vector::size_type size_type;
+
+  public:
+
+    typedef std::forward_iterator_tag iterator_category;
+    typedef size_type value_type;
+    typedef size_type difference_type;
+    typedef size_type* pointer;
+    typedef size_type& reference;
+
+    const_bv_iterator(const ITERABLE_BV* p_iterable, size_type pos)
+      : p_iterable_(const_cast<ITERABLE_BV*>(p_iterable)), pos_(pos)
+    {}
+
+    bool operator!= (const const_bv_iterator &other) const{
+      return pos_ < other.pos_;
+    }
+
+    size_type operator *() const{
+      return size_type(*p_iterable_);
+    }
+
+    //difference of iterators
+    size_type operator-(const const_bv_iterator &other) const{
+      if (pos_ == other.pos_) return 0;
+
+      auto &vector_this  = p_iterable_->v_;
+      auto &vector_other = other.p_iterable_->v_;
+      bv_visitor v_this(vector_this), v_other(vector_other);
+      while (v_this < pos_) ++v_this;
+      while (v_other < other.pos_) ++v_other;
+      auto &v     = (pos_ < other.pos_)  ? v_this : v_other;
+      auto &v_end = (pos_ >= other.pos_) ? v_this : v_other;
+
+      size_type count = 0;
+      while(v < v_end) { ++v; ++count;}
+      return count;
+    }
+
+    const const_bv_iterator &operator++(){
+      ++*p_iterable_;
+      pos_ = *p_iterable_;
+      return *this;
+    }
+
+  private:    
+    ITERABLE_BV *p_iterable_;
+    size_type pos_;
+  };
+
+    
+  /**
+  Wrapper class to make bit_vector iterable on true values.
+  It enables the use of c++11 range-based loop feature.
+  example:
+  @code
+  for (auto i : bv_iterable(bit_vector)) {
+  .... //(use i as an unsigned int)
+  }
+  */
+  class bv_iterable
+  {
+  public:
+    bv_iterable(const bit_vector &v) : v_(v), visitor_(v){}
+
+    const_bv_iterator<bv_iterable> begin() const;
+    const_bv_iterator<bv_iterable> end() const;
+    inline bool operator++(){return ++visitor_;};
+    inline operator dal::bit_vector::size_type() const{return visitor_;};
+
+  private:
+    const bit_vector& v_;
+    bv_visitor visitor_;
+    friend class const_bv_iterator<bv_iterable>;
+  };
+
+  /***Same as bv_iterable class except the bit_vector is copied locally.*/
+  class bv_iterable_c
+  {
+  public:
+    bv_iterable_c(const bit_vector &v) : v_(v), visitor_(v){}
+    const_bv_iterator<bv_iterable_c> begin() const;
+    const_bv_iterator<bv_iterable_c> end() const;
+    inline bool operator++(){return ++visitor_;};
+    inline operator dal::bit_vector::size_type() const{return visitor_;};
+
+  private:
+    bit_vector v_;
+    bv_visitor visitor_;
+    friend class const_bv_iterator<bv_iterable_c>;
+  };
 
 }
 
