@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2000-2012 Yves Renard, Julien Pommier
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2000-2015 Yves Renard, Julien Pommier
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /** @file getfem_assembling.h
@@ -42,8 +42,6 @@
 #define GETFEM_ASSEMBLING_H__
 
 #include "getfem_assembling_tensors.h"
-#include "getfem/getfem_mesh_im_level_set.h"
-
 
 namespace getfem {
   
@@ -67,7 +65,7 @@ namespace getfem {
     v.push_back(w[0]);
     w.resize(2);
     MPI_SUM_VECTOR(v, w);
-    return v[1]/v[0];
+    return w[1]/w[0];
   }
 
   /**
@@ -1136,7 +1134,7 @@ namespace getfem {
    const VECT2 &r_data, const mesh_region &region,
    int version =  ASMDIR_BUILDALL) {
     typedef typename gmm::linalg_traits<VECT1>::value_type value_type;
-    typedef typename gmm::number_traits<value_type>::magnitude_type magn_type;
+    // typedef typename gmm::number_traits<value_type>::magnitude_type magn_type;
 
     if ((version & ASMDIR_SIMPLIFY) &&
 	(mf_u.is_reduced() || mf_mult.is_reduced() || mf_r.is_reduced())) {
