@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2011-2013 Yves Renard, Konstantinos Poulios.
+ Copyright (C) 2011-2015 Yves Renard, Konstantinos Poulios.
 
- This file is a part of GETFEM++
+ This file is a part of GetFEM++
 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -38,20 +38,12 @@
 #ifndef GETFEM_CONTACT_AND_FRICTION_COMMON_H__
 #define GETFEM_CONTACT_AND_FRICTION_COMMON_H__
 
-#include "getfem_models.h"
+#include "getfem_generic_assembly.h"
 #include "getfem_assembling_tensors.h"
 #include "getfem/bgeot_rtree.h"
 #include <getfem/getfem_mesher.h>
 
 
-#include <getfem/getfem_arch_config.h>
-#if GETFEM_HAVE_MUPARSER_MUPARSER_H
-#include <muParser/muParser.h>
-#elif GETFEM_HAVE_MUPARSER_H
-#include <muParser.h>
-#endif
-
-
 namespace getfem {
 
   //=========================================================================
@@ -410,8 +402,11 @@ namespace getfem {
   //  Structure which stores the contact boundaries, rigid obstacles and
   //  computes the contact pairs in large sliding/large deformation
   //
+  //  Deprecated
+  //
   //=========================================================================
 
+
   class multi_contact_frame {
 
     // Structure describing a contact boundary
@@ -471,10 +466,9 @@ namespace getfem {
     std::vector<contact_boundary> contact_boundaries;
 
     std::vector<std::string> coordinates;
-    base_node pt_eval;
-#if GETFEM_HAVE_MUPARSER_MUPARSER_H || GETFEM_HAVE_MUPARSER_H
-    std::vector<mu::Parser> obstacles_parsers;
-#endif
+    model_real_plain_vector pt, ptx, pty, ptz, ptw;
+    std::list<ga_workspace> obstacles_gw;
+    std::vector<ga_function> obstacles_f;
     std::vector<std::string> obstacles;
     std::vector<std::string> obstacles_velocities;
 
@@ -798,7 +792,7 @@ namespace getfem {
   /** Add a rigid obstacle whose geometry corresponds to the zero level-set
       of the high-level generic assembly expression `expr`
       to an existing raytracing interpolate transformation called 'transname'.
-      In `expr`, the current position is denoted `X` whith components
+      In `expr`, the current position is denoted `X` with components
       `X(1), X(2), ...`. It is also allowed to use `x` instead of `X(1)`,
       `y` instead of `X(2)`, `z` instead of `X(3)` and `w` instead of `X(4)`. 
   */
