@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2013-2014 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2013-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /** @file getfem_generic_assembly.h
@@ -36,7 +36,7 @@
  */
 
 
-#ifndef GETFEM_GENERIC_ASSEMBLY_H__ 
+#ifndef GETFEM_GENERIC_ASSEMBLY_H__
 #define GETFEM_GENERIC_ASSEMBLY_H__
 
 #include <map>
@@ -46,6 +46,9 @@
 
 #ifdef _WIN32
 #include <limits>
+#if defined(INFINITY)
+#undef INFINITY
+#endif
 #define INFINITY std::numeric_limits<scalar_type>::infinity()
 #endif
 
@@ -53,6 +56,8 @@ namespace getfem {
 
   class ga_tree;
 
+  int ga_check_name_validity(const std::string &name);
+
   //=========================================================================
   // Structure dealing with predefined operators.
   //=========================================================================
@@ -65,7 +70,7 @@ namespace getfem {
                              bgeot::multi_index &sizes) const = 0;
 
     virtual void value(const arg_list &args, base_tensor &result) const = 0;
-    
+
     virtual void derivative(const arg_list &args, size_type i,
                             base_tensor &result) const = 0;
 
@@ -78,7 +83,7 @@ namespace getfem {
   struct ga_predef_operator_tab {
     typedef std::map<std::string, ga_nonlinear_operator*> T;
     std::map<std::string, ga_nonlinear_operator*> tab;
-    
+
    void add_method(const std::string &name, ga_nonlinear_operator *pt)
     { tab[name] = pt; }
     ~ga_predef_operator_tab() {
@@ -90,24 +95,19 @@ namespace getfem {
   //=========================================================================
   // For user predefined scalar functions.
   //=========================================================================
-  
+
   typedef scalar_type (*pscalar_func_onearg)(scalar_type);
   typedef scalar_type (*pscalar_func_twoargs)(scalar_type, scalar_type);
-  struct ga_interval {
-    scalar_type min, max;
-    ga_interval(void) { min = -INFINITY; max = +INFINITY; }
-    ga_interval(scalar_type a, scalar_type b) { min = a; max = b; }
-  };
 
   void ga_define_function(const std::string name, size_type nb_args,
                           const std::string expr, const std::string der1="",
                           const std::string der2="");
   void ga_define_function(const std::string name, pscalar_func_onearg f,
-                          const std::string der1="");
+                          const std::string &der1="");
   void ga_define_function(const std::string name, pscalar_func_twoargs f2,
                           const std::string &der1="",
                           const std::string &der2="");
- 
+
   void ga_undefine_function(const std::string name);
   bool ga_function_exists(const std::string name);
 
@@ -117,9 +117,10 @@ namespace getfem {
   //=========================================================================
 
   class ga_workspace {
-    
+
     const model *md;
     const ga_workspace *parent_workspace;
+    bool enable_all_md_variables;
 
     struct var_description {
 
@@ -129,16 +130,27 @@ namespace getfem {
       gmm::sub_interval I;
       const model_real_plain_vector *V;
       const im_data *imd;
+      bgeot::multi_index qdims;  // For data having a qdim != of the fem
+                                 // (dim per dof for dof data)
+                                 // and for constant variables.
 
-      var_description(bool is_var,
-                      bool is_fem, 
-                      const mesh_fem *mmf,
-                      gmm::sub_interval I_,
-                      const model_real_plain_vector *v, const im_data *imd_)
+      size_type qdim(void) const {
+        size_type q = 1;
+        for (size_type i = 0; i < qdims.size(); ++i) q *= qdims[i];
+        return q;
+      }
+
+      var_description(bool is_var, bool is_fem,
+                      const mesh_fem *mmf, gmm::sub_interval I_,
+                      const model_real_plain_vector *v, const im_data *imd_,
+                      size_type Q)
         : is_variable(is_var), is_fem_dofs(is_fem), mf(mmf), I(I_), V(v),
-          imd(imd_) {}
+          imd(imd_), qdims(1) {
+        GMM_ASSERT1(Q > 0, "Bad dimension");
+        qdims[0] = Q;
+      }
       var_description() : is_variable(false), is_fem_dofs(false),
-                          mf(0), V(0), imd(0) {}
+                          mf(0), V(0), imd(0), qdims(1) { qdims[0] = 1; }
     };
 
   public:
@@ -159,19 +171,23 @@ namespace getfem {
       ~tree_description();
     };
 
+    static const mesh dummy_mesh;
+    static const mesh_im dummy_mim;
+    static const mesh_region dummy_region;
+
   private:
 
+    mutable std::map<std::string, gmm::sub_interval> int_disabled_variables;
+
     std::map<const mesh *, std::list<mesh_region> > registred_mims;
-    mesh dummy_mesh;
-    mesh dummy_mim;
-    mesh_region dummy_region;
-    
-    mesh_region &register_region(const mesh &m, const mesh_region &region);
+
+    const mesh_region &register_region(const mesh &m,const mesh_region &region);
 
     typedef std::map<std::string, var_description> VAR_SET;
 
     VAR_SET variables;
     std::map<std::string, pinterpolate_transformation> transformations;
+    std::map<std::string, pelementary_transformation> elem_transformations;
     std::vector<tree_description> trees;
     std::list<ga_tree *> aux_trees;
 
@@ -185,13 +201,13 @@ namespace getfem {
       m_tree(void) : ptree(0) {}
       ~m_tree(void);
     };
-    
+
     mutable std::map<std::string, m_tree> macro_trees;
 
 
     void add_tree(ga_tree &tree, const mesh &m, const mesh_im &mim,
                   const mesh_region &rg,
-                  const std::string &expr, bool add_derivative = true,
+                  const std::string &expr, size_type add_derivative_order = 2,
                   bool scalar_expr = true);
     void clear_aux_trees(void);
 
@@ -266,56 +282,53 @@ namespace getfem {
     model_real_sparse_matrix &unreduced_matrix(void)
     { return unreduced_K; }
     base_vector &unreduced_vector(void) { return unreduced_V; }
-    
+
     /** Add an expression, perform the semantic analysis, split into
      *  terms in separated test functions, derive if necessary to obtain
      *  the tangent terms. Return the maximal order found in the expression.
      */
     size_type add_expression(const std::string expr, const mesh_im &mim,
-                            const mesh_region &rg=mesh_region::all_convexes());
+                             const mesh_region &rg=mesh_region::all_convexes(),
+                             size_type add_derivative_order = 2);
     /* Internal use */
     void add_function_expression(const std::string expr);
     /* Internal use */
     void add_interpolation_expression(const std::string expr, const mesh &m,
-                                      mesh_region rg);
+                                      const mesh_region &rg=mesh_region::all_convexes());
+    void add_interpolation_expression(const std::string expr, const mesh_im &mim,
+                                      const mesh_region &rg=mesh_region::all_convexes());
 
     /** Delete all previously added expressions. */
     void clear_expressions(void);
-    
+
+    /** Print some information about all previously added expressions. */
+    void print(std::ostream &str);
 
     void add_aux_tree(ga_tree &tree);
     size_type nb_trees(void) const;
     tree_description &tree_info(size_type i);
-        
+
     void add_fem_variable(const std::string &name, const mesh_fem &mf,
                           const gmm::sub_interval &I,
-                          const model_real_plain_vector &VV)
-    { variables[name] = var_description(true, true, &mf, I, &VV, 0); }
-    
+                          const model_real_plain_vector &VV);
     void add_fixed_size_variable(const std::string &name,
                                  const gmm::sub_interval &I,
-                                 const model_real_plain_vector &VV)
-    { variables[name] = var_description(true, false, 0, I, &VV, 0); }
-
+                                 const model_real_plain_vector &VV);
     void add_fem_constant(const std::string &name, const mesh_fem &mf,
-                          const model_real_plain_vector &VV) { 
-      variables[name] = var_description(false, true, &mf,
-                                        gmm::sub_interval(), &VV, 0);
-    }
-    
+                          const model_real_plain_vector &VV);
     void add_fixed_size_constant(const std::string &name,
-                                 const model_real_plain_vector &VV) {
-      variables[name] = var_description(false, false, 0,
-                                        gmm::sub_interval(), &VV, 0);
-    }
-
+                                 const model_real_plain_vector &VV);
     void add_im_data(const std::string &name, const im_data &imd,
-                     const model_real_plain_vector &VV) {
-      variables[name] = var_description(false, false, 0,
-                                        gmm::sub_interval(), &VV, &imd);
-    }
+                     const model_real_plain_vector &VV);
 
-    bool used_variables(model::varnamelist &vl, model::varnamelist &dl,
+    std::string extract_constant_term(const mesh &m);
+    std::string extract_order1_term(const std::string &varname);
+    std::string extract_order0_term();
+    std::string extract_Neumann_term(const std::string &varname);
+
+
+    bool used_variables(model::varnamelist &vl, model::varnamelist &vl_test1,
+                        model::varnamelist &vl_test2, model::varnamelist &dl,
                         size_type order);
 
     bool variable_exists(const std::string &name) const {
@@ -358,7 +371,7 @@ namespace getfem {
       GMM_ASSERT1(t.size(), "Variable group " << name << " has no variable");
       return t[0];
     }
-    
+
     bool macro_exists(const std::string &name) const {
       if (macros.find(name) != macros.end()) return true;
       if (md && md->macro_exists(name)) return true;
@@ -369,7 +382,7 @@ namespace getfem {
 
     void add_macro(const std::string &name, const std::string &expr)
     { macros[name] = expr; }
-   
+
     const std::string& get_macro(const std::string &name) const {
       std::map<std::string, std::string>::const_iterator it=macros.find(name);
       if (it != macros.end()) return it->second;
@@ -381,7 +394,7 @@ namespace getfem {
     }
 
     ga_tree &macro_tree(const std::string &name, size_type meshdim,
-                        bool ignore_X) const;
+                        size_type ref_elt_dim, bool ignore_X) const;
 
     void add_interpolate_transformation(const std::string &name,
                                         pinterpolate_transformation ptrans)
@@ -407,23 +420,82 @@ namespace getfem {
       GMM_ASSERT1(false, "Inexistent transformation " << name);
     }
 
+    void add_elementary_transformation(const std::string &name,
+                                       pelementary_transformation ptrans)
+    { elem_transformations[name] = ptrans; }
+
+    bool elementary_transformation_exists(const std::string &name) const {
+      return (md && md->elementary_transformation_exists(name)) ||
+        (parent_workspace &&
+         parent_workspace->elementary_transformation_exists(name)) ||
+        (elem_transformations.find(name) != elem_transformations.end());
+    }
+
+    pelementary_transformation
+    elementary_transformation(const std::string &name) const {
+      std::map<std::string, pelementary_transformation>::const_iterator
+        it = elem_transformations.find(name);
+      if (it != elem_transformations.end()) return it->second;
+      if (md && md->elementary_transformation_exists(name))
+        return md->elementary_transformation(name);
+      if (parent_workspace &&
+         parent_workspace->elementary_transformation_exists(name))
+        return parent_workspace->elementary_transformation(name);
+      GMM_ASSERT1(false, "Inexistent elementary transformation " << name);
+    }
+
     bool is_constant(const std::string &name) const {
       VAR_SET::const_iterator it = variables.find(name);
       if (it != variables.end()) return !(it->second.is_variable);
       if (variable_group_exists(name))
         return is_constant(first_variable_of_group(name));
-      if (md && md->variable_exists(name)) return md->is_data(name);
+      if (md && md->variable_exists(name)) {
+        if (enable_all_md_variables) return md->is_true_data(name);
+        return md->is_data(name);
+      }
       if (parent_workspace && parent_workspace->variable_exists(name))
         return parent_workspace->is_constant(name);
       GMM_ASSERT1(false, "Undefined variable " << name);
     }
 
+    bool is_disabled_variable(const std::string &name) const {
+      VAR_SET::const_iterator it = variables.find(name);
+      if (it != variables.end()) return false;
+      if (variable_group_exists(name))
+        return is_disabled_variable(first_variable_of_group(name));
+      if (md && md->variable_exists(name)) {
+        if (enable_all_md_variables) return false;
+        return md->is_disabled_variable(name);
+      }
+      if (parent_workspace && parent_workspace->variable_exists(name))
+        return parent_workspace->is_disabled_variable(name);
+      GMM_ASSERT1(false, "Undefined variable " << name);
+    }
+
+    const scalar_type &factor_of_variable(const std::string &name) const {
+      static const scalar_type one(1);
+      VAR_SET::const_iterator it = variables.find(name);
+      if (it != variables.end()) return one;
+      if (variable_group_exists(name))
+        return one;
+      if (md && md->variable_exists(name)) return md->factor_of_variable(name);
+      if (parent_workspace && parent_workspace->variable_exists(name))
+        return parent_workspace->factor_of_variable(name);
+      GMM_ASSERT1(false, "Undefined variable " << name);
+    }
+
+    const gmm::sub_interval &
+    interval_of_disabled_variable(const std::string &name) const;
+
     const gmm::sub_interval &
     interval_of_variable(const std::string &name) const {
       VAR_SET::const_iterator it = variables.find(name);
       if (it != variables.end()) return it->second.I;
-      if (md && md->variable_exists(name))
+      if (md && md->variable_exists(name)) {
+        if (enable_all_md_variables && md->is_disabled_variable(name))
+          return interval_of_disabled_variable(name);
         return md->interval_of_variable(name);
+      }
       if (parent_workspace && parent_workspace->variable_exists(name))
         return parent_workspace->interval_of_variable(name);
       GMM_ASSERT1(false, "Undefined variable " << name);
@@ -453,51 +525,26 @@ namespace getfem {
       GMM_ASSERT1(false, "Undefined variable " << name);
     }
 
-    size_type qdim(const std::string &name) const {
-      const mesh_fem *mf = associated_mf(name);
-      const im_data *imd = associated_im_data(name);
-      size_type n = gmm::vect_size(value(name));
-      if (mf) {
-        size_type ndof = mf->nb_dof();
-        GMM_ASSERT1(ndof, "Variable " << name << " with no dof. You probably "
-                    "make a wrong initialization of a mesh_fem object");
-        return mf->get_qdim() * (n / ndof);
-      } else if (imd) {
-        size_type q = n / imd->nb_filtered_index();
-        GMM_ASSERT1(q % imd->nb_tensor_elem() == 0,
-                    "Invalid mesh im data vector");
-        return q;
-      }
-      return n;
-    }
+    size_type qdim(const std::string &name) const;
+//     {
+//       const mesh_fem *mf = associated_mf(name);
+//       const im_data *imd = associated_im_data(name);
+//       size_type n = ;
+//       if (mf) {
+//         size_type ndof = mf->nb_dof();
+//         GMM_ASSERT1(ndof, "Variable " << name << " with no dof. You probably "
+//                     "made a wrong initialization of a mesh_fem object");
+//         return mf->get_qdim() * (n / ndof);
+//       } else if (imd) {
+//         size_type q = n / imd->nb_filtered_index();
+//         GMM_ASSERT1(q % imd->nb_tensor_elem() == 0,
+//                     "Invalid mesh im data vector");
+//         return q;
+//       }
+//       return gmm::vect_size(value(name));
+//     }
 
-    bgeot::multi_index qdims(const std::string &name) const {
-      const mesh_fem *mf = associated_mf(name);
-      const im_data *imd = associated_im_data(name);
-      size_type n = gmm::vect_size(value(name));
-      if (mf) {
-        size_type ndof = mf->nb_dof();
-        GMM_ASSERT1(ndof, "Variable " << name << " with no dof. You probably "
-                    "make a wrong initialization of a mesh_fem object");
-        bgeot::multi_index mi = mf->get_qdims();
-        size_type qmult = n / ndof;
-        if (qmult > 1) {
-          if (mi.back() == 1) mi.back() *= qmult; else mi.push_back(qmult);
-        }
-        return mi;
-      } else if (imd) {
-        bgeot::multi_index mi = imd->tensor_size();
-        size_type q = n / imd->nb_filtered_index();
-        GMM_ASSERT1(q % imd->nb_tensor_elem() == 0,
-                    "Invalid mesh im data vector");
-        size_type qmult = q / imd->nb_tensor_elem();
-        if (qmult > 1) {
-          if (mi.back() == 1) mi.back() *= qmult; else mi.push_back(qmult);
-        }
-        return mi;
-      }
-      bgeot::multi_index mi(1); mi[0] = n; return mi;
-    }
+    bgeot::multi_index qdims(const std::string &name) const;
 
     const model_real_plain_vector &value(const std::string &name) const {
       VAR_SET::const_iterator it = variables.find(name);
@@ -514,9 +561,13 @@ namespace getfem {
 
     void assembly(size_type order);
 
-    ga_workspace(const getfem::model &md_) : md(&md_), parent_workspace(0) {}
-    ga_workspace(const ga_workspace &gaw) : md(0), parent_workspace(&gaw) {}
-    ga_workspace(void) : md(0), parent_workspace(0) {}
+    ga_workspace(const getfem::model &md_, bool enable_all_variables = false)
+      : md(&md_), parent_workspace(0),
+        enable_all_md_variables(enable_all_variables) {}
+    ga_workspace(bool, const ga_workspace &gaw)
+      : md(0), parent_workspace(&gaw), enable_all_md_variables(false) {}
+    ga_workspace(void)
+      : md(0), parent_workspace(0), enable_all_md_variables(false) {}
     ~ga_workspace() { clear_expressions(); }
 
   };
@@ -533,7 +584,7 @@ namespace getfem {
     mutable ga_workspace local_workspace;
     std::string expr;
     mutable ga_instruction_set *gis;
-    
+
   public:
     ga_function(void) : gis(0) {}
     ga_function(const model &md, const std::string &e);
@@ -560,42 +611,41 @@ namespace getfem {
     points_for_element(size_type cv, short_type f,
                        std::vector<size_type> &ind) const = 0;
     virtual bool use_pgp(size_type cv) const = 0;
+    virtual bool use_mim(void) const = 0;
     virtual void store_result(size_type cv, size_type i, base_tensor &t) = 0;
     virtual void finalize(void) = 0;
     virtual const mesh &linked_mesh(void) = 0;
     virtual ~ga_interpolation_context() {}
   };
-  
+
 
   //=========================================================================
   // Interpolation functions
   //=========================================================================
 
-  
+
   void ga_interpolation(ga_workspace &workspace,
                         ga_interpolation_context &gic);
 
   void ga_interpolation_Lagrange_fem
   (ga_workspace &workspace, const mesh_fem &mf, base_vector &result);
-  
+
   void ga_interpolation_Lagrange_fem
   (const getfem::model &md, const std::string &expr, const mesh_fem &mf,
    base_vector &result, const mesh_region &rg=mesh_region::all_convexes());
 
-  // Not tested
   void ga_interpolation_mti
   (const getfem::model &md, const std::string &expr, mesh_trans_inv &mti,
    base_vector &result, const mesh_region &rg=mesh_region::all_convexes(),
    int extrapolation = 0, const mesh_region &rg_source=mesh_region::all_convexes(),
    size_type nbdof_ = size_type(-1));
 
-  // Not tested
   void ga_interpolation_im_data
-  (const getfem::model &md, const std::string &expr, im_data &imd,
+  (const getfem::model &md, const std::string &expr, const im_data &imd,
    base_vector &result, const mesh_region &rg=mesh_region::all_convexes());
 
   //=========================================================================
-  // Interpolation transformation
+  // Interpolate transformations
   //=========================================================================
 
   /** Add a transformation to the model `md` from mesh `source_mesh` to mesh
@@ -603,7 +653,7 @@ namespace getfem {
       high-level generic assembly expression which may contains some
       variable of the model. CAUTION: For the moment, the derivative of the
       transformation with respect to the eventual variables used is not
-      taken into account in the model solve. 
+      taken into account in the model solve.
   */
   void add_interpolate_transformation_from_expression
   (model &md, const std::string &transname, const mesh &source_mesh,
@@ -612,7 +662,7 @@ namespace getfem {
   void add_interpolate_transformation_from_expression
   (ga_workspace &workspace, const std::string &transname,
    const mesh &source_mesh, const mesh &target_mesh, const std::string &expr);
-  
+
 }  /* end of namespace getfem.                                             */
 
 
