@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 1999-2012 Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 1999-2015 Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_mesh.h
@@ -66,8 +66,8 @@ namespace getfem {
   /**@addtogroup mesh*/
   /**@{*/
 
-  /** Describe a mesh (collection of convexes and points).  Note that
-      mesh object have no copy constructor, use
+  /** Describe a mesh (collection of convexes (elements) and points).
+      Note that mesh object have no copy constructor, use
       mesh::copy_from instead.  This class inherits from
       bgeot::mesh<base_node> and bgeot::mesh_structure. Compared to
       bgeot::mesh, It provides some additional methods for convenience
@@ -101,8 +101,8 @@ namespace getfem {
   */
 
   class APIDECL mesh : virtual public dal::static_stored_object,
-		       public bgeot::basic_mesh,
-		       public context_dependencies 
+                       public bgeot::basic_mesh,
+                       public context_dependencies 
 
 //allowing mesh::shared_from_this() call
 #if defined(__GNUC__) 
@@ -134,7 +134,7 @@ namespace getfem {
     mutable std::map<size_type, mesh_region> cvf_sets;
     mutable dal::bit_vector valid_cvf_sets;
     void handle_region_refinement(size_type, const std::vector<size_type> &,
-				  bool);
+                                  bool);
 
     mutable bool cuthill_mckee_uptodate;
     dal::dynamic_array<gmm::uint64_type> cvs_v_num;
@@ -199,8 +199,8 @@ namespace getfem {
 
     /// Return a (pseudo)container of points of face of a given convex
     ref_mesh_face_pt_ct points_of_face_of_convex(size_type ic,
-						 size_type f) const {
-      ind_pt_face_ct rct = ind_points_of_face_of_convex(ic, short_type(f));
+                                                 short_type f) const {
+      ind_pt_face_ct rct = ind_points_of_face_of_convex(ic, f);
       return ref_mesh_face_pt_ct(pts.begin(), rct.begin(), rct.end());
     }
 
@@ -209,40 +209,40 @@ namespace getfem {
     { return ref_convex(structure_of_convex(ic), points_of_convex(ic)); }
 
     /** Add the point pt to the mesh and return the index of the
-	point.
+        point.
 
-	If the point is too close to an existing point, the
-	function does not create a new point, and returns the index of the
-	already existing point.
-	@param pt the point coordinates.
+        If the point is too close to an existing point, the
+        function does not create a new point, and returns the index of the
+        already existing point.
+        @param pt the point coordinates.
     */
     size_type add_point(const base_node &pt) { return pts.add_node(pt); }
-    //			scalar_type characteristic_size = scalar_type(1));
+    //                        scalar_type characteristic_size = scalar_type(1));
 
     /// Give the number of geometrical nodes in the mesh.
     size_type nb_points(void) const { return pts.card(); }
     /// Return the points index
     const dal::bit_vector &points_index(void) const { return pts.index(); }
     /** Delete the point of index i from the mesh if it is not linked to a
-	convex.
+        convex.
     */
     void sup_point(size_type i) { if (!is_point_valid(i)) pts.sup_node(i); }
     /// Swap the indexes of points of index i and j in the whole structure.
     void swap_points(size_type i, size_type j)
     { if (i != j) { pts.swap_points(i,j); mesh_structure::swap_points(i,j); } }
     /** Search a point given its coordinates.
-	@param pt the point that is searched.
-	@return the point index if pt was found in (or approximatively in)
-	the mesh nodes, size_type(-1) if not found.
+        @param pt the point that is searched.
+        @return the point index if pt was found in (or approximatively in)
+        the mesh nodes, size_type(-1) if not found.
     */
     size_type search_point(const base_node &pt, const scalar_type radius=0) const
     { return pts.search_node(pt,radius); }
     /** Return the bgeot::geometric_trans attached to a convex.
-	@param ic the convex number.
+        @param ic the convex number.
     */
     bgeot::pgeometric_trans trans_of_convex(size_type ic) const {
       GMM_ASSERT1(trans_exists[ic],
-		  "No geometric transformation or nonexisting element");
+                  "No geometric transformation or nonexisting element");
       return gtab[ic];
     }
 
@@ -251,31 +251,31 @@ namespace getfem {
     { return cvs_v_num[ic]; }
 
     /** Add a convex to the mesh.
-	This methods assume that the convex nodes have already been
-	added to the mesh.
-	@param pgt the geometric transformation of the convex.
-	@param ipts an iterator to a set of point index.
-	@return the number of the new convex.
+        This methods assume that the convex nodes have already been
+        added to the mesh.
+        @param pgt the geometric transformation of the convex.
+        @param ipts an iterator to a set of point index.
+        @return the number of the new convex.
      */
     template<class ITER>
     size_type add_convex(bgeot::pgeometric_trans pgt, ITER ipts) {
       bool present;
       size_type i = bgeot::mesh_structure::add_convex(pgt->structure(),
-						       ipts, &present);
+                                                       ipts, &present);
       gtab[i] = pgt; trans_exists[i] = true;
       if (!present) { cvs_v_num[i] = act_counter(); touch(); }
       return i;
     }
 
     /** Add a convex to the mesh, given a geometric transformation and a
-	list of point coordinates.
+        list of point coordinates.
 
-	As a side-effect, the points are also added to the mesh (if
-	they were not already in the mesh).
+        As a side-effect, the points are also added to the mesh (if
+        they were not already in the mesh).
 
-	@param pgt the geometric transformation of the convex.
-	@param ipts an iterator on a set of getfem::base_node.
-	@return the number of the new convex.
+        @param pgt the geometric transformation of the convex.
+        @param ipts an iterator on a set of getfem::base_node.
+        @return the number of the new convex.
     */
     template<class ITER>
     size_type add_convex_by_points(bgeot::pgeometric_trans pgt, ITER ipts);
@@ -286,7 +286,7 @@ namespace getfem {
     size_type add_simplex(dim_type di, ITER ipts)
     { return add_convex(bgeot::simplex_geotrans(di, 1), ipts); }
     /** Add a simplex to the mesh, given its dimension and point coordinates.
-	@see add_convex_by_points.
+        @see add_convex_by_points.
      */
     template<class ITER>
     size_type add_simplex_by_points(dim_type dim, ITER ipts);
@@ -294,7 +294,7 @@ namespace getfem {
     size_type add_segment(size_type a, size_type b);
     /** Add a segment to the mesh, given the coordinates of its vertices. */
     size_type add_segment_by_points(const base_node &pt1,
-				    const base_node &pt2) {
+                                    const base_node &pt2) {
       size_type ipt1 = add_point(pt1);
       size_type ipt2 = add_point(pt2);
       return add_segment(ipt1, ipt2);
@@ -303,27 +303,27 @@ namespace getfem {
     size_type add_triangle(size_type a,size_type b, size_type c);
     /** Add a triangle to the mesh, given the coordinates of its vertices. */
     size_type add_triangle_by_points(const base_node &p1,
-				     const base_node &p2,
-				     const base_node &p3);
+                                     const base_node &p2,
+                                     const base_node &p3);
     /** Add a tetrahedron to the mesh, given the point id of its vertices. */
     size_type add_tetrahedron(size_type a,
-			      size_type b, size_type c, size_type d);
+                              size_type b, size_type c, size_type d);
     /** Add a tetrahedron to the mesh, given the coordinates of its vertices. */
     size_type add_tetrahedron_by_points(const base_node &p1,
-					const base_node &p2,
-					const base_node &p3,
-					const base_node &p4);
+                                        const base_node &p2,
+                                        const base_node &p3,
+                                        const base_node &p4);
     /** Add a parallelepiped to the mesh.
-	@param di dimension of the parallelepiped
-	@param ipts iterator on the list of point id.
-	@return the number of the new convex.
+        @param di dimension of the parallelepiped
+        @param ipts iterator on the list of point id.
+        @return the number of the new convex.
     */
     template<class ITER>
     size_type add_parallelepiped(dim_type di, const ITER &ipts);
     /** Add a parallelepiped to the mesh.
-	@param di dimension of the parallelepiped
-	@param ps iterator on the list of point coordinates.
-	@return the number of the new convex.
+        @param di dimension of the parallelepiped
+        @param ps iterator on the list of point coordinates.
+        @return the number of the new convex.
     */
     template<class ITER>
     size_type add_parallelepiped_by_points(dim_type di, const ITER &ps);
@@ -335,20 +335,20 @@ namespace getfem {
 
     template<class ITER>
     size_type add_parallelepiped_by_vectors(dim_type di,
-				  const base_node &org, const ITER &vects);
+                                  const base_node &org, const ITER &vects);
      */
     /** Add a prism to the mesh.
-	@param di dimension of the prism
-	@param ipts iterator on the list of point id.
-	@return the number of the new convex.
+        @param di dimension of the prism
+        @param ipts iterator on the list of point id.
+        @return the number of the new convex.
     */
     template<class ITER>
     size_type add_prism(dim_type di, const ITER &ipts);
 
     /** Add a prism to the mesh.
-	@param di dimension of the prism
-	@param ps iterator on the list of point coordinates.
-	@return the number of the new convex.
+        @param di dimension of the prism
+        @param ps iterator on the list of point coordinates.
+        @return the number of the new convex.
     */
     template<class ITER>
     size_type add_prism_by_points(dim_type di, const ITER &ps);
@@ -367,17 +367,17 @@ namespace getfem {
     void swap_convex(size_type i, size_type j);
 
     /** Return the normal of the given convex face, evaluated at the point pt.
-	@param ic the convex number.
-	@param f the face number.
-	@param pt the point at which the normal is taken, in the
-	reference convex. This point should of course be on the
-	correspounding face of the reference convex, except if the
-	geometric transformation is linear: in that case, the normal
-	constant.
-	@return the face normal.
+        @param ic the convex number.
+        @param f the face number.
+        @param pt the point at which the normal is taken, in the
+        reference convex. This point should of course be on the
+        correspounding face of the reference convex, except if the
+        geometric transformation is linear: in that case, the normal
+        constant.
+        @return the face normal.
     */
     base_small_vector normal_of_face_of_convex(size_type ic, short_type f,
-					       const base_node &pt) const;
+                                               const base_node &pt) const;
     /* Return the normal of the given convex face.
 
        @param ic the convex number.
@@ -387,7 +387,7 @@ namespace getfem {
        @return the face normal.
     */
     base_small_vector normal_of_face_of_convex(size_type ic, short_type f,
-					       size_type n=0) const;
+                                               size_type n=0) const;
 
 
     /* Return the mean outward unit normal vector of the given convex
@@ -407,7 +407,7 @@ namespace getfem {
        tangent vectors in the other columns.
     */
     base_matrix local_basis_of_face_of_convex(size_type ic, short_type f,
-					      const base_node &pt) const;
+                                              const base_node &pt) const;
     /** Return a local basis for the convex face.
        @param ic the convex number.
        @param f the face number.
@@ -416,13 +416,13 @@ namespace getfem {
        tangent vectors in the other columns.
     */
     base_matrix local_basis_of_face_of_convex(size_type ic, short_type f,
-					      size_type n) const;
+                                              size_type n) const;
     /** Return an estimate of the convex quality (0 <= Q <= 1). */
     scalar_type convex_quality_estimate(size_type ic) const;
     /** Return an estimate of the convex area.
-	@param ic the convex number
-	@param degree the degree of the approximation integration
-	method used to compute the area.
+        @param ic the convex number
+        @param degree the degree of the approximation integration
+        method used to compute the area.
     */
     scalar_type convex_area_estimate(size_type ic, size_type degree=2) const;
     /** Return an estimate of the convex largest dimension. @see getfem::convex_quality_estimate */
@@ -438,9 +438,9 @@ namespace getfem {
     /** Return the bounding box [Pmin - Pmax] of the mesh. */
     void bounding_box(base_node& Pmin, base_node &Pmax) const;
     /** Return the region of index 'id'. Regions stored in mesh are
-	updated when the mesh is modified (i.e. when a convex is
-	removed from the mesh, it is also removed from all regions of
-	the mesh.
+        updated when the mesh is modified (i.e. when a convex is
+        removed from the mesh, it is also removed from all regions of
+        the mesh.
     */
     const mesh_region region(size_type id) const {
       if (id == mesh_region::all_convexes().id())
@@ -470,34 +470,34 @@ namespace getfem {
     /** Remove the region of index b. */
     void sup_region(size_type b) {
       if (valid_cvf_sets[b])
-	{ cvf_sets[b].clear(); valid_cvf_sets.sup(b); touch(); }
+        { cvf_sets[b].clear(); valid_cvf_sets.sup(b); touch(); }
     }
     /** Remove all references to a convex from all regions stored in the mesh.
      @param cv the convex number.*/
     void sup_convex_from_regions(size_type cv);
     /** Pack the mesh : renumber convexes and nodes such that there
-	is no holes in their numbering. Do NOT do the Cuthill-McKee. */
+        is no holes in their numbering. Do NOT do the Cuthill-McKee. */
     void optimize_structure(void);
     /// Return the list of convex IDs for a Cuthill-McKee ordering
     const std::vector<size_type> &cuthill_mckee_ordering() const;
     /// Erase the mesh.
     void clear(void);
     /** Write the mesh to a file. The format is getfem-specific.
-	@param name the file name.
+        @param name the file name.
     */
     void write_to_file(const std::string &name) const;
     /** Write the mesh to a text stream.
-	@param ost the stream.
+        @param ost the stream.
     */
     void write_to_file(std::ostream &ost) const;
     /** Load the mesh from a file.
-	@param name the file name.
-	@see getfem::import_mesh.
+        @param name the file name.
+        @see getfem::import_mesh.
     */
     void read_from_file(const std::string &name);
     /** Load the mesh from a text stream.
-	@param ist the text stream.
-	@see getfem::import_mesh.
+        @param ist the text stream.
+        @see getfem::import_mesh.
     */
     void read_from_file(std::istream &ist);
     /** Clone a mesh */
@@ -548,16 +548,16 @@ namespace getfem {
     void set_name(const std::string&);
 
     void Bank_convex_with_edge(size_type, size_type,
-			       std::vector<size_type> &);
+                               std::vector<size_type> &);
     bool Bank_is_convex_having_points(size_type,
-				      const std::vector<size_type> &);
+                                      const std::vector<size_type> &);
     void Bank_sup_convex_from_green(size_type);
     void Bank_swap_convex(size_type, size_type);
     void Bank_build_first_mesh(mesh &, size_type);
     void Bank_basic_refine_convex(size_type);
     void Bank_refine_normal_convex(size_type);
     size_type Bank_test_and_refine_convex(size_type, dal::bit_vector &,
-					  bool = true);
+                                          bool = true);
     void Bank_build_green_simplexes(size_type, std::vector<size_type> &);
 
   public :
@@ -574,11 +574,12 @@ namespace getfem {
   /**
    * build a N+1 dimensions mesh from a N-dimensions mesh by extrusion.
    */
-  void APIDECL extrude(const mesh& in, mesh& out, unsigned nb_layers);
+  void APIDECL extrude(const mesh& in, mesh& out, size_type nb_layers,
+                       short_type degree=short_type(1));
 
  template<class ITER>
     size_type mesh::add_convex_by_points(bgeot::pgeometric_trans pgt,
-					                           ITER ipts)
+                                                                   ITER ipts)
   {
     short_type nb = short_type(pgt->nb_points());
     std::vector<size_type> ind(nb);
@@ -617,34 +618,34 @@ namespace getfem {
       of the convex area.
    */
   scalar_type APIDECL convex_area_estimate(bgeot::pgeometric_trans pgt,
-				   const base_matrix& pts,
-				   pintegration_method pim);
+                                   const base_matrix& pts,
+                                   pintegration_method pim);
 
   /** rough estimate of the maximum value of the condition
    * number of the jacobian of the geometric transformation */
   scalar_type APIDECL convex_quality_estimate(bgeot::pgeometric_trans pgt,
-				      const base_matrix& pts);
+                                      const base_matrix& pts);
 
   /** rough estimate of the radius of the convex using the largest eigenvalue
    * of the jacobian of the geometric transformation */
   scalar_type APIDECL convex_radius_estimate(bgeot::pgeometric_trans pgt,
-				     const base_matrix& pts);
+                                     const base_matrix& pts);
 
   /* stores a convex face. if f == -1, it is the whole convex.             */
   struct convex_face;
   typedef std::vector<convex_face> convex_face_ct;
   struct APIDECL convex_face  {
     size_type cv;
-    size_type f;
+    short_type f;
     inline bool operator < (const convex_face &e) const
     {
       if (cv < e.cv) return true; if (cv > e.cv) return false;
       if (f < e.f) return true; else if (f > e.f) return false;
       return false;
     }
-    bool is_face() const { return f != size_type(-1); }
-    convex_face(size_type cv_, size_type f_=size_type(-1)) : cv(cv_), f(f_) {}
-    convex_face() : cv(size_type(-1)), f(size_type(-1)) {}
+    bool is_face() const { return f != short_type(-1); }
+    convex_face(size_type cv_, short_type f_=short_type(-1)) : cv(cv_), f(f_) {}
+    convex_face() : cv(size_type(-1)), f(short_type(-1)) {}
   }; //  IS_DEPRECATED;
 
   /** returns a list of "exterior" faces of a mesh
@@ -652,7 +653,7 @@ namespace getfem {
    * + convexes whose dimension is smaller that m.dim()
    */
   void  APIDECL outer_faces_of_mesh(const mesh &m, const dal::bit_vector& cvlst,
-			    convex_face_ct& flist);
+                            convex_face_ct& flist);
 
   inline void APIDECL outer_faces_of_mesh(const mesh &m, convex_face_ct& flist)
     IS_DEPRECATED;
@@ -660,7 +661,7 @@ namespace getfem {
   { outer_faces_of_mesh(m,m.convex_index(),flist); }
 
   void  APIDECL outer_faces_of_mesh(const mesh &m, const mesh_region &cvlst,
-			    mesh_region &flist);
+                            mesh_region &flist);
 
   inline void  APIDECL outer_faces_of_mesh(const mesh &m, mesh_region &flist)
   { outer_faces_of_mesh(m,m.convex_index(),flist); }
@@ -671,10 +672,18 @@ namespace getfem {
 
   /** Select in the region mr the faces of the mesh m with their unit
       outward vector having a maximal angle "angle" with the vector V.
-   */ 
-  mesh_region APIDECL select_faces_of_normal(const mesh &m, const mesh_region &mr,
-                                     const base_small_vector &V,
-                                     scalar_type angle);
+   */
+  mesh_region APIDECL select_faces_of_normal(const mesh &m,
+                                             const mesh_region &mr,
+                                             const base_small_vector &V,
+                                             scalar_type angle);
+
+  /** Select in the region mr the faces of the mesh m lying entirely in the
+      box delimated by pt1 and pt2.
+   */
+  mesh_region APIDECL select_faces_in_box(const mesh &m, const mesh_region &mr,
+                                          const base_node &pt1,
+                                          const base_node &pt2);
 
 
   ///@}
