@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
- 
- Copyright (C) 2004-2012 Julien Pommier, Yves Renard
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2004-2015 Julien Pommier, Yves Renard
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -17,7 +17,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
  As a special exception, you  may use  this file  as it is a part of a free
  software  library  without  restriction.  Specifically,  if   other  files
  instantiate  templates  or  use macros or inline functions from this file,
@@ -26,7 +26,7 @@
  to be covered  by the GNU Lesser General Public License.  This   exception
  does not  however  invalidate  any  other  reasons why the executable file
  might be covered by the GNU Lesser General Public License.
- 
+
 ===========================================================================*/
 
 /**@file getfem_mesher.h
@@ -141,7 +141,7 @@ namespace getfem {
     bool bounding_box(base_node &, base_node &) const
     { return false; }
     virtual scalar_type operator()(const base_node &P) const
-    {  return base.eval(P.begin()) + shift_ls; }
+    {  return bgeot::to_scalar(base.eval(P.begin())) + shift_ls; }
     virtual scalar_type operator()(const base_node &P,
 				   dal::bit_vector &bv) const
     { scalar_type d = (*this)(P); bv[id] = (gmm::abs(d) < SEPS); return d; }
@@ -388,7 +388,7 @@ namespace getfem {
   public:
     mesher_union(const std::vector<const mesher_signed_distance *>
 			&dists_) : dists(dists_) 
-    { vd.resize(dists.size()); }
+    { vd.resize(dists.size()); with_min = true; }
 
     mesher_union(const mesher_signed_distance &a_,
 		 const mesher_signed_distance &b_,
@@ -537,7 +537,7 @@ namespace getfem {
 	dists[i]->hess(P, H);
       }
       else {
-	GMM_ASSERT1(false, "Sorry, to e done");
+	GMM_ASSERT1(false, "Sorry, to be done");
       }
     }
   };
