@@ -1,10 +1,10 @@
 /*===========================================================================
- 
- Copyright (C) 2004-2012 Julien Pommier
- 
- This file is a part of GETFEM++
- 
- Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+
+ Copyright (C) 2004-2015 Julien Pommier
+
+ This file is a part of GetFEM++
+
+ GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -16,7 +16,7 @@
  You  should  have received a copy of the GNU Lesser General Public License
  along  with  this program;  if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
- 
+
 ===========================================================================*/
 
 #include "getfem/getfem_superlu.h"
@@ -174,7 +174,8 @@ namespace gmm {
 
     SuperMatrix SA, SL, SU, SB, SX; // SuperLU format.
     Create_CompCol_Matrix(&SA, m, n, nz, const_cast<T*>(&csc_A.pr[0]),
-			  (int *)(&csc_A.ir[0]), (int *)(&csc_A.jc[0]));
+			  const_cast<int *>((const int *)(&csc_A.ir[0])),
+                          const_cast<int *>((const int *)(&csc_A.jc[0])));
     Create_Dense_Matrix(&SB, m, nrhs, &rhs[0], m);
     Create_Dense_Matrix(&SX, m, nrhs, &sol[0], m);
 
@@ -285,7 +286,8 @@ namespace gmm {
     StatInit(&stat);
     
     Create_CompCol_Matrix(&SA, m, n, nz, const_cast<T*>(&A.pr[0]),
-                          (int *)(&A.ir[0]), (int *)(&A.jc[0]));
+                          const_cast<int *>((const int *)(&A.ir[0])),
+                          const_cast<int *>((const int *)(&A.jc[0])));
     Create_Dense_Matrix(&SB, m, 0, &rhs[0], m);
     Create_Dense_Matrix(&SX, m, 0, &sol[0], m);
     memset(&SL,0,sizeof SL);
