@@ -1,15 +1,16 @@
-# Copyright (C) 2001-2009 Yves Renard
+# Copyright (C) 2001-2015 Yves Renard
 #
-# This file is a part of GETFEM++
+# This file is a part of GetFEM++
 #
-# Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+# GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
 # under  the  terms  of the  GNU  Lesser General Public License as published
-# by  the  Free Software Foundation;  either version 2.1 of the License,  or
-# (at your option) any later version.
+# by  the  Free Software Foundation;  either version 3 of the License,  or
+# (at your option) any later version along with the GCC Runtime Library
+# Exception either version 3.1 or (at your option) any later version.
 # This program  is  distributed  in  the  hope  that it will be useful,  but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 # or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-# License for more details.
+# License and GCC Runtime Library Exception for more details.
 # You  should  have received a copy of the GNU Lesser General Public License
 # along  with  this program;  if not, write to the Free Software Foundation,
 # Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -21,7 +22,7 @@ sub catch { `rm -f $tmp`; exit(1); }
 $SIG{INT} = 'catch';
 
 open(TMPF, ">$tmp") or die "Open file impossible : $!\n";
-print TMPF <<
+print TMPF <<""
 LX =100;   % size in X in mm.        %2.0; %1.0;		
 LY =20;   % size in Y in mm.        %0.5; %1.0;
 LZ =20;   % size in Z in mm.        %0.5;
